/*
   Teensy Color Organ
     File: GUI_Elements.h - Purpose: Defines all on screen widgets

     Color Organ Concept, Design and Implementation by: Craig A. Lindley
     Version: 0.5
     Last Update: 09/12/2016

   Copyright (c) 2016, Craig A. Lindley -  calhjh@gmail.com

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice, development funding notice, and this permission
   notice shall be included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/

#ifndef GUI_ELEMENTS
#define GUI_ELEMENTS

// Max chars in TextField
#define MAX_TEXT_LENGTH 40

// Draw the title and border around the display
void drawBorder(void) {

  lcd.fillScreen(ILI9341_BLACK);

  // Set big font
  lcd.setTextSize(2);
  lcd.setTextColor(ILI9341_BLUE, ILI9341_BLACK);

  // Print header
  printCenteredString("Teensy Color Organ", 0, 0, 320, 16);
  lcd.drawLine(0, 17, 319, 17, ILI9341_BLUE);

  // Set small font
  lcd.setTextSize(1);

  // Print footer
  lcd.drawLine(0, 223, 319, 223, ILI9341_BLUE);
  printCenteredString("Copyright 2016 - Craig A. Lindley", 0, 224, 320, 16);

  // Draw vertical lines
  lcd.drawLine(  0, 18,   0, 223, ILI9341_BLUE);
  lcd.drawLine(319, 18, 319, 223, ILI9341_BLUE);
}

// Basic class backing all GUI fields and controls
class GUI_Element {

  public:
    // Public methods
    GUI_Element(int _x, int _y, int _w, int _h,
                uint16_t _textColor, uint16_t _backgroundColor, uint16_t _borderColor,
                const char *_text) {
      // Save incoming
      x = _x;
      y = _y;
      w = _w;
      h = _h;
      textColor = _textColor;
      backgroundColor = _backgroundColor;
      borderColor = _borderColor;
      strncpy(text, _text, MAX_TEXT_LENGTH);
    }

  protected:
    // Protected data
    int x, y, w, h;
    uint16_t textColor, backgroundColor, borderColor;
    char text[MAX_TEXT_LENGTH];

    virtual void paint(void) = 0;
};

// TextField GUI element
class GUI_TextField : public GUI_Element {
  public:
    GUI_TextField(int _x, int _y, int _w, int _h,
                  uint16_t _textColor, uint16_t _backgroundColor, uint16_t _borderColor,
                  const char *_text) :
      GUI_Element(_x, _y, _w, _h,  _textColor, _backgroundColor, _borderColor, _text) {}

    // Paint the text field on the display
    void paint(void) {

      lcd.fillRect(x, y, w, h, backgroundColor);
      lcd.drawRect(x, y, w, h, borderColor);
      lcd.setTextColor(textColor);
      printCenteredString(text, x, y, w, h);
    }

    // Fill the text field with color
    void colorTextField(uint16_t color) {
      lcd.fillRect(x, y, w, h, color);
    }

    // Set a new value into the text field
    void setText(const char *_text) {

      // Copy new text
      strncpy(text, _text, MAX_TEXT_LENGTH);

      // Repaint the field
      paint();
    }
};

// VUField GUI element
class GUI_VUField : public GUI_Element {
  public:
    GUI_VUField(int _x, int _y, int _w, int _h,
                uint16_t _barColor, uint16_t _backgroundColor, uint16_t _borderColor) :
      GUI_Element(_x, _y, _w, _h, _barColor, _backgroundColor, _borderColor, "") {

      // NOTE: cannot call clearVUMeter because LCD is not yet initialized
      // Clear array of values used in smoothing
      memset(values, 0, sizeof(values));

      readIndex = 0;  // the index of the current reading
      total = 0;      // the running total
      average = 0;    // the average
    }

    // Paint the VU field on the display
    void paint(void) {

      // Draw border
      lcd.drawRect(x, y, w, h, borderColor);

      // Next draw the full bar
      lcd.fillRect(x + 2, y + 2, w - 4, h - 4, textColor);

      // Finally erase portions of the bar leaving level
      lcd.fillRect(x + 2, y + 2, w - 4, (h - 5) * (1 - average), backgroundColor);
    }

    // Clear the VU meter
    void clearVUMeter(void) {

      // Clear array of values used in smoothing
      memset(values, 0, sizeof(values));

      readIndex = 0;  // the index of the current reading
      total = 0;      // the running total
      average = 0;    // the average

      // Repaint the field
      paint();
    }

    // Set a new value into the VU field
    void setValue(float value) {
      // Perform some smoothing on the VU data
      // Subtract the last reading:
      total -= values[readIndex];

      // Store new value
      values[readIndex] = value;

      // Add new value to the total:
      total += value;

      // Advance to the next position in the array:
      readIndex++;

      // Check for wrap around
      readIndex = (readIndex >= MAX_VALUES) ? 0 : readIndex;

      // calculate the average:
      average = total / MAX_VALUES;

      // Repaint the field
      paint();
    }

  private:
    static const int MAX_VALUES = 15;
    int readIndex = 0;  // the index of the current reading
    float total = 0;    // the running total
    float average = 0;  // the average
    float values[MAX_VALUES];
};

// Button GUI element
class GUI_Button : public GUI_Element {
  public:
    GUI_Button(int _x, int _y, int _w, int _h,
               uint16_t _textColor, uint16_t _backgroundColor, uint16_t _borderColor,
               const char *_text) :
      GUI_Element(_x, _y, _w, _h, _textColor, _backgroundColor, _borderColor, _text) {}

    // Paint the button on the screen
    void paint(void) {


      lcd.fillRect(x, y, w, h, backgroundColor);
      lcd.drawRect(x, y, w, h, borderColor);
      lcd.setTextColor(textColor);
      printCenteredString(text, x, y, w, h);
    }

    // Determine if the button has been touched
    bool touched(int touchX, int touchY) {
      // Is this the button clicked ?
      if (((touchX >= x) && (touchX <= x + w)) &&
          ((touchY >= y) && (touchY <= y + h))) {
        // Button touched
        return true;
      } else  {
        return false;
      }
    }
};

// Dimensions of the Value Picker component parts
#define VP_TF_WIDTH  150  // Width of text field between the buttons
#define VP_TF_HEIGHT  30  // Height of text field between the buttons
#define VP_B_WIDTH    30  // Width of inc/dec buttons
#define VP_B_HEIGHT   VP_TF_HEIGHT  // Height of inc/dec buttons
#define VP_TF_B_SPACE 20  // Space between buttons and text field
#define VP_X_OFFSET ((320 - VP_TF_WIDTH - (2 * VP_B_WIDTH) - (2 * VP_TF_B_SPACE)) / 2)
#define VP_INC_B_X  (VP_X_OFFSET)
#define VP_TF_X     (VP_X_OFFSET + VP_B_WIDTH + VP_TF_B_SPACE)
#define VP_DEC_B_X  (VP_TF_X + VP_TF_WIDTH + VP_TF_B_SPACE)

// ValuePicker GUI element
class GUI_ValuePicker {
  public:
    GUI_ValuePicker(int _y, uint16_t _textColor, uint16_t _backgroundColor, uint16_t _borderColor) {

      // Dynamically create the incrementer button
      pIncrementer =
        new GUI_Button(VP_INC_B_X, _y, VP_B_WIDTH, VP_B_HEIGHT, _textColor, _backgroundColor, _borderColor, "+");

      // Dynamically create the text field for value display
      pTextField =
        new GUI_TextField(VP_TF_X, _y, VP_TF_WIDTH, VP_TF_HEIGHT, _textColor, _backgroundColor, _borderColor, "");

      // Dynamically create the decrementer button
      pDecrementer =
        new GUI_Button(VP_DEC_B_X, _y, VP_B_WIDTH, VP_B_HEIGHT, _textColor, _backgroundColor, _borderColor, "-");
    }

    // Paint the button on the screen
    void paint(void) {

      lcd.setTextSize(2);

      pIncrementer->paint();
      pTextField->paint();
      pDecrementer->paint();
    }

    void setTextFieldText(const char *text) {
      pTextField->setText(text);
    }

    void setTextFieldColor(uint16_t color) {
      pTextField->colorTextField(color);
    }

    bool incrementerTouched(int touchX, int touchY) {
      return pIncrementer->touched(touchX, touchY);
    }

    bool decrementerTouched(int touchX, int touchY) {
      return pDecrementer->touched(touchX, touchY);
    }

  private:
    GUI_TextField *pTextField;
    GUI_Button *pIncrementer;
    GUI_Button *pDecrementer;
};

#endif

