/*
   Teensy Color Organ
     File: LEDOutputControl.h - Purpose: Control of the LEDs for each band

     Color Organ Concept, Design and Implementation by: Craig A. Lindley
     Version: 0.5
     Last Update: 09/12/2016

   Copyright (c) 2016, Craig A. Lindley - calhjh@gmail.com

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice, development funding notice, and this permission
   notice shall be included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/

extern Adafruit_NeoPixel ledStrip;

#ifndef LEDOUTPUTCONTROL
#define LEDOUTPUTCONTROL

class LEDOutputControl {

  public:

    // Class Constructor
    LEDOutputControl(void) {
      clearBitSet();
    }

    // Fill the Band_LEDs array with the indices of the LEDs currently assigned for display
    // Density is specified in %
    void assignBandLEDs(void) {

      int ledPick;
      int index;
      bool done;

      // Calculate the number of LEDs per band from the density
      int numberOfLeds = round(((float) configurationData.ledDensity * LEDS_PER_BAND) / 100.0);

      for (int band = 0; band < NUMBER_OF_BANDS; band++) {

        // Remove all previous LED assignments for band
        for (int i = 0; i < LEDS_PER_BAND; i++) {
          Band_LEDs[band][i] = 255;
        }

        index = 0;

        for (int i = 0; i < numberOfLeds; i++) {
          done = false;
          while (! done) {
            // Pick an LED randomly
            ledPick = random(NUMBER_OF_LEDS);
            // Have we picked this LED before ?
            if (! testBit(ledPick)) {
              // No so indicate it has been picked
              setBit(ledPick);
              Band_LEDs[band][index++] = ledPick;
              done = true;
            }
          }
        }
      }
    }

    // Assign LEDs for all bands based on density
    void assignLEDsBasedOnDensity(void) {

      // Clear previous band assignments
      clearBitSet();

      // Turn off all LEDs before reassignment
      ledStrip.clear();
      ledStrip.show();

      assignBandLEDs();
    }

    // Set Band LEDs to a color value
    void setBandLEDsToColorValue(int band, int value) {

      int ledPick, index = 0;
      bool done = false;
      while (! done) {
        ledPick = Band_LEDs[band][index++];
        if (ledPick == 255) {
          done = true;
        } else  {
          ledStrip.setPixelColor(ledPick, Band_Colors[band][value]);
        }
      }
      ledStrip.show();
    }

    // Assign colors for each band. Each band is represented by a hue value. From the band's
    // hue value we calculate the 256 values of V in HSV color model
    void calculateBandColors(void) {

      int hue = 0;
      int hueInc = round(256.0 / NUMBER_OF_BANDS);

      for (int band = 0; band < NUMBER_OF_BANDS - 1; band++) {
        for (int value = 0; value < 256; value++) {
          Band_Colors[band][value] = create24BitColor(hue, value);
        }
        hue += hueInc;
      }
    }

  private:

    // LED assignments for each band
    byte Band_LEDs[NUMBER_OF_BANDS][LEDS_PER_BAND];

    // 256 colors for each band
    uint32_t Band_Colors[NUMBER_OF_BANDS][256];

    // These 32 bit variables make up a bit set with 1 bit for each of the 120 LEDs
    uint32_t bitSet0;
    uint32_t bitSet1;
    uint32_t bitSet2;
    uint32_t bitSet3;

    // Clear all of the bits in the bit set
    void clearBitSet(void) {
      bitSet0 = bitSet1 = bitSet2 = bitSet3 = 0;
    }

    // Test a bit in the bit set
    bool testBit(int bitNumber) {

      uint32_t set = 0;

      int setNumber = bitNumber / 32;
      int setOffset = bitNumber % 32;

      switch (setNumber) {
        case 0:
          set = bitSet0;
          break;
        case 1:
          set = bitSet1;
          break;
        case 2:
          set = bitSet2;
          break;
        case 3:
          set = bitSet3;
          break;
      }
      return (bitRead(set, setOffset) == 1);
    }

    // Set a bit in the bit set
    void setBit(int bitNumber) {

      int setNumber = bitNumber / 32;
      int setOffset = bitNumber % 32;

      switch (setNumber) {
        case 0:
          bitWrite(bitSet0, setOffset, 1);
          break;
        case 1:
          bitWrite(bitSet1, setOffset, 1);
          break;
        case 2:
          bitWrite(bitSet2, setOffset, 1);
          break;
        case 3:
          bitWrite(bitSet3, setOffset, 1);
          break;
      }
    }
};

#endif

