/*
   Teensy Color Organ
     File: PrintExtensions.h - Purpose: Code for formatting text strings

     Color Organ Concept, Design and Implementation by: Craig A. Lindley
     Version: 0.5
     Last Update: 09/12/2016

   Copyright (c) 2016, Craig A. Lindley - calhjh@gmail.com

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice, development funding notice, and this permission
   notice shall be included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/

extern ILI9341_t3 lcd;

#ifndef PRINTEXT_H
#define PRINTEXT_H

// Hardcoded font dimensions
#define FONT_WIDTH  5
#define FONT_HEIGHT 7

// Horizontal and Vertical alignment enumerations
enum HORIZ_ALIGNMENT {
  HLEFT, HCENTER, HRIGHT
};

enum VERT_ALIGNMENT {
  VTOP, VMIDDLE, VBOTTOM
};

/************************************************************************/
/*                        Print Extension Functions                     */
/************************************************************************/

// Specialize string printing function
void printAlignedString(const char *string, int x, int y, int width, int height, HORIZ_ALIGNMENT hAlign, VERT_ALIGNMENT vAlign) {

  int length = strlen(string);

  // Determine font dimensions from stock 5x7 font
  int factor = lcd.getTextSize();
  int fontWidth  = (FONT_WIDTH  + 1) * factor;
  int fontHeight = (FONT_HEIGHT + 1) * factor;

  int aX = x;
  int aY = y;

  // Process horizontal alignment
  switch (hAlign) {
    case HLEFT:
      // Nothing to do
      break;

    case HCENTER:
      aX = x + (width / 2) - ((length * fontWidth) / 2);
      break;

    case HRIGHT:
      aX = x + width - (length * fontWidth);
      break;
  }

  // Process vertical alignment
  switch (vAlign) {
    case VTOP:
      // Nothing to do
      break;

    case VMIDDLE:
      aY = y + (height / 2) - (fontHeight / 2);
      break;

    case VBOTTOM:
      aY = y + height - fontHeight;
      break;
  }
  // Print the string
  lcd.setCursor(aX, aY);
  lcd.print(string);
}

// Prints string centered both horizontally and vertically in specified area
void printCenteredString(const char *string, int x, int y, int width, int height) {

  printAlignedString(string, x, y, width, height, HCENTER, VMIDDLE);
}

// Prints an integer centered both horizontally and vertically in specified area
void printCenteredInt(int i, int x, int y, int width, int height) {

  char buffer[10];

  // Convert int to string in buffer
  itoa(i, buffer, 10);

  printCenteredString(buffer, x, y, width, height);
}

// Helper function to clear field and print new int value
void clearAndPrintCenteredInt(int i, int x, int y, int width, int height) {

  // Print the centered integer
  printCenteredInt(i, x, y, width, height);
}

// Helper function to clear field and print new string value
void clearAndPrintCenteredString(char *string, int x, int y, int width, int height) {

  // Print the centered string
  printCenteredString(string, x, y, width, height);
}

#endif






