/*
   Teensy Color Organ
     File: Screen_Files.h - Purpose: Screen for saving/loading color organ configurations

     Color Organ Concept, Design and Implementation by: Craig A. Lindley
     Version: 0.5
     Last Update: 09/12/2016

   Copyright (c) 2016, Craig A. Lindley - calhjh@gmail.com

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice, development funding notice, and this permission
   notice shall be included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/

extern void configureColorOrgan(void);

#ifndef FILESSCREEN
#define FILESSCREEN

#define FILE1_NAME "f1.cfg"
#define FILE2_NAME "f2.cfg"
#define FILE3_NAME "f3.cfg"
#define FILE4_NAME "f4.cfg"

// Instantiate the buttons
GUI_Button f1Button( 12, 52, 65, 40, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE, "File1");
GUI_Button f2Button( 89, 52, 65, 40, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE, "File2");
GUI_Button f3Button(166, 52, 65, 40, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE, "File3");
GUI_Button f4Button(243, 52, 65, 40, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE, "File4");
GUI_Button storeButton(20, 115, 80, 40, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE, "Store");
GUI_Button fbackButton(120, 115, 80, 40, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE, "Back");
GUI_Button loadButton(220, 115, 80, 40, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE, "Load");

// Instantiate the status text field
GUI_TextField statusField(10, 178, 300, 30, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE, "");

class FilesScreen : public BaseScreen {

  public:
    // Initialize the screen
    void init(void) {
      // Clear the screen and draw the border
      drawBorder();

      lcd.setTextSize(2);

      // Make the buttons visible
      f1Button.paint();
      f2Button.paint();
      f3Button.paint();
      f4Button.paint();
      storeButton.paint();
      fbackButton.paint();
      loadButton.paint();

      // Clear the text field
      statusField.setText("Touch file to begin");

      // Make the text field visible
      statusField.paint();

      // Clear the filename buffer
      memset(buffer, 0, sizeof(buffer));
    }

    // Run the screen's code
    enum STATES run(void) {

      // Has the screen been touched
      if (readTouch()) {

        // Determine which button if any was touched
        // Has the back button been touched ?
        if (fbackButton.touched(touchX, touchY)) {
          return CONFIG_SCR_INIT_STATE;

        } else if (f1Button.touched(touchX, touchY)) {
          checkFile(FILE1_NAME);

        } else if (f2Button.touched(touchX, touchY)) {
          checkFile(FILE2_NAME);

        } else if (f3Button.touched(touchX, touchY)) {
          checkFile(FILE3_NAME);

        } else if (f4Button.touched(touchX, touchY)) {
          checkFile(FILE4_NAME);

        } else if (storeButton.touched(touchX, touchY)) {
          // Was a file previously specified ?
          if (fileSpecified()) {
            // Yes so attempt to store the config data
            storeConfigurationDataFile();
          }

        } else if (loadButton.touched(touchX, touchY)) {
          // Was a file previously specified ?
          if (fileSpecified()) {
            // Yes so attempt to store the config data
            loadConfigurationDataFile();
          }
        }
      }
      return FILES_SCR_STATE;
    }

  protected:

    void checkFile(const char *filename) {

      // Keep track of which file button was touched
      strcpy(buffer, filename);

      // See if file exists
      if (SD.exists(buffer)) {
        statusField.setText("File exists !");
      } else  {
        statusField.setText("File does not exist !");
      }
    }

    // Make sure a file has been specified
    bool fileSpecified(void) {
      if (buffer[0] == '\0') {
        statusField.setText("No file selected !");
        return false;
      } else  {
        return true;
      }
    }

    // Store the configuration data to the current file
    void storeConfigurationDataFile(void) {

      // Open the current file
      File configFile = SD.open(buffer, FILE_WRITE);

      // Did the file open successfully
      if (configFile) {
        // Seek to the beginning of the file to overwrite previous content
        configFile.seek(0);

        // Now attempt to write the file
        byte bytesWritten = configFile.write((byte *) &configurationData, CONFIG_DATA_SIZE);
        if (bytesWritten == CONFIG_DATA_SIZE) {
          // File written successfully
          statusField.setText("File store successful !");
        } else  {
          statusField.setText("File store failed !");
        }
        // Close the open file in either case
        configFile.close();
        return;
      }
      statusField.setText("File store failed !");
    }

    // Load the configuration data from the current file
    void loadConfigurationDataFile(void) {

      // Open the current file
      File configFile = SD.open(buffer, FILE_READ);

      // Did the file open successfully
      if (configFile) {
        // File open succeeded so attempt to read the file
        byte bytesRead = configFile.read((byte *) &configurationData, CONFIG_DATA_SIZE);
        if (bytesRead == CONFIG_DATA_SIZE) {
          // File read successfully
          statusField.setText("File load successful !");

          // Configure color organ to match configuration data just
          // loaded from file
          configureColorOrgan();

        } else  {
          statusField.setText("File load failed !");
        }
        // Close the open file in either case
        configFile.close();
        return;
      }
      statusField.setText("File load failed !");
    }

  private:
    char buffer[12];
};

#endif

