/*
   Teensy Color Organ
     File: Screen_Misc.h - Purpose: Screen and controls for setting miscellaneous color organ parameters

     Color Organ Concept, Design and Implementation by: Craig A. Lindley
     Version: 0.5
     Last Update: 09/12/2016

   Copyright (c) 2016, Craig A. Lindley - calhjh@gmail.com

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice, development funding notice, and this permission
   notice shall be included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
*/

extern void configureColorOrgan(void);

#ifndef MISCSCREEN
#define MISCSCREEN

#define MIN_DYNAMICS_MINS   0
#define MAX_DYNAMICS_MINS 120
#define DYNAMICS_INC        2

// Instantiate value pickers
GUI_ValuePicker inputPicker(36, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE);
GUI_ValuePicker avcPicker(85, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE);
GUI_ValuePicker dynamicsPicker(135, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE);

// Instantiate button
GUI_Button mBackButton(120, 175, 80, 40, ILI9341_BLACK, ILI9341_GREEN, ILI9341_BLUE, "Back");

class MiscScreen : public BaseScreen {

  public:

    // Initialize the screen
    void init(void) {

      // Clear the screen and draw the border
      drawBorder();

      // Set small font
      lcd.setTextSize(1);
      lcd.setTextColor(ILI9341_WHITE);

      // Draw screen labels
      printCenteredString("Input Selector", 0, 20, 320, 16);
      printCenteredString("Automatic Volume Control", 0, 68, 320, 16);
      printCenteredString("Dynamics Interval Minutes", 0, 118, 320, 16);

      // Install default values
      inputPicker.setTextFieldText(configurationData.micInput ? "Mic" : "Line");
      avcPicker.setTextFieldText(configurationData.avcOn ? "On" : "Off");
      dynamicsPicker.setTextFieldText(convertIntToString(configurationData.dynamicsIntervalMins));

      // Make screen controls visible
      inputPicker.paint();
      avcPicker.paint();
      dynamicsPicker.paint();
      mBackButton.paint();
    }

    enum STATES run(void) {

      // Has the screen been touched
      if (readTouch()) {

        // Has the back button been touched ?
        if (mBackButton.touched(touchX, touchY)) {
          // Yes return to configuration screen
          return CONFIG_SCR_INIT_STATE;
        }

        // Was the input picker incrementer touched ?
        if (inputPicker.incrementerTouched(touchX, touchY)) {
          if (configurationData.micInput == false) {
            configurationData.micInput = true;
            audioController.inputSelect(configurationData.micInput ? AUDIO_INPUT_MIC : AUDIO_INPUT_LINEIN);
          }
        }
        // Was the input picker decrementer touched ?
        if (inputPicker.decrementerTouched(touchX, touchY)) {
          if (configurationData.micInput == true) {
            configurationData.micInput = false;
            audioController.inputSelect(configurationData.micInput ? AUDIO_INPUT_MIC : AUDIO_INPUT_LINEIN);
          }
        }

        // Was the avc picker incrementer touched ?
        if (avcPicker.incrementerTouched(touchX, touchY)) {
          if (configurationData.avcOn == false) {
            configurationData.avcOn = true;
            configurationData.avcOn ? audioController.autoVolumeEnable() : audioController.autoVolumeDisable();
          }
        }
        // Was the avc picker decrementer touched ?
        if (avcPicker.decrementerTouched(touchX, touchY)) {
          if (configurationData.avcOn == true) {
            configurationData.avcOn = false;
            configurationData.avcOn ? audioController.autoVolumeEnable() : audioController.autoVolumeDisable();
          }
        }

        // Was the dynamics incrementer touched ?
        if (dynamicsPicker.incrementerTouched(touchX, touchY)) {
          configurationData.dynamicsIntervalMins += DYNAMICS_INC;
          if (configurationData.dynamicsIntervalMins > MAX_DYNAMICS_MINS) {
            configurationData.dynamicsIntervalMins = MAX_DYNAMICS_MINS;
          }
        }
        // Was the dynamics picker decrementer touched ?
        if (dynamicsPicker.decrementerTouched(touchX, touchY)) {
          configurationData.dynamicsIntervalMins -= DYNAMICS_INC;
          if (configurationData.dynamicsIntervalMins <  MIN_DYNAMICS_MINS) {
            configurationData.dynamicsIntervalMins = MIN_DYNAMICS_MINS;
          }
        }

        // Update value pickers text fields
        inputPicker.setTextFieldText(configurationData.micInput ? "Mic" : "Line");
        avcPicker.setTextFieldText(configurationData.avcOn ? "On" : "Off");
        dynamicsPicker.setTextFieldText(convertIntToString(configurationData.dynamicsIntervalMins));
      }
      // Stay on this screen until back buttons is touched
      return MISC_SCR_STATE;
    }
};

#endif

